@@ -7,6 +7,7 @@ from BT1.btformats import check_message
 from BT1.Choker import Choker
 from BT1.Storage import Storage
 from BT1.StorageWrapper import StorageWrapper
+from BT1.FileSelector import FileSelector
 from BT1.Uploader import Upload
 from BT1.Downloader import Downloader
 from BT1.HTTPDownloader import HTTPDownloader
@@ -20,14 +21,16 @@ from RateMeasure import RateMeasure
 from CurrentRateMeasure import Measure
 from BT1.PiecePicker import PiecePicker
 from BT1.Statistics import Statistics
+from ConfigDir import ConfigDir
 from bencode import bencode, bdecode
+from natpunch import UPnP_test
 from sha import sha
 from os import path, makedirs, listdir
 from parseargs import parseargs, formatDefinitions, defaultargs
 from socket import error as socketerror
 from random import seed
 from threading import Thread, Event
-from time import time
+from clock import clock
 from __init__ import createPeerID
 
 try:
@@ -51,12 +54,22 @@ defaults = [
         "maximum length prefix encoding you'll accept over the wire - larger values get the connection dropped."),
     ('ip', '',
         "ip to report you have to the tracker."),
-    ('minport', 6881, 'minimum port to listen on, counts up if unavailable'),
-    ('maxport', 6999, 'maximum port to listen on'),
+    ('minport', 10000, 'minimum port to listen on, counts up if unavailable'),
+    ('maxport', 60000, 'maximum port to listen on'),
+    ('random_port', 1, 'whether to choose randomly inside the port range ' +
+        'instead of counting up linearly'),
     ('responsefile', '',
         'file the server response was stored in, alternative to url'),
     ('url', '',
         'url to get file from, alternative to responsefile'),
+    ('selector_enabled', 1,
+        'whether to enable the file selector and fast resume function'),
+    ('expire_cache_data', 10,
+        'the number of days after which you wish to expire old cache data ' +
+        '(0 = disabled)'),
+    ('priority', '',
+        'a list of file priorities separated by commas, must be one per file, ' +
+        '0 = highest, 1 = normal, 2 = lowest, -1 = download disabled'),
     ('saveas', '',
         'local file name to save the file as, null indicates query user'),
     ('timeout', 300.0,
@@ -74,10 +87,14 @@ defaults = [
          'allow the client to connect to peers via IPv6'),
     ('ipv6_binds_v4', autodetect_socket_style(),
         'set if an IPv6 server socket will also field IPv4 connections'),
-    ('upnp_nat_access', 0,
-        'attempt to autoconfigure a UPnP router to forward a server port'),
+    ('upnp_nat_access', 1,
+        'attempt to autoconfigure a UPnP router to forward a server port ' +
+        '(0 = disabled, 1 = mode 1 [fast], 2 = mode 2 [slow])'),
     ('upload_rate_fudge', 5.0, 
         'time equivalent of writing to kernel-level TCP buffer, for rate adjustment'),
+    ('tcp_ack_fudge', 0.03,
+        'how much TCP ACK download overhead to add to upload rate calculations ' +
+        '(0 = disabled)'),
     ('display_interval', .5,
         'time between updates of displayed information'),
     ('rerequest_interval', 5 * 60,
@@ -91,7 +108,9 @@ defaults = [
     ('check_hashes', 1,
         'whether to check hashes on disk'),
     ('max_upload_rate', 0,
-        'maximum kB/s to upload at, 0 means no limit'),
+        'maximum kB/s to upload at (0 = no limit, -1 = automatic)'),
+    ('max_download_rate', 0,
+        'maximum kB/s to download at (0 = no limit)'),
     ('alloc_type', 'normal',
         'allocation type (may be normal, background, pre-allocate or sparse)'),
     ('alloc_rate', 2.0,
@@ -131,6 +150,8 @@ defaults = [
         "whether to lock files the client is working with"),
     ('lock_while_reading', 0,
         "whether to lock access to files being read"),
+    ('auto_flush', 0,
+        "minutes between automatic flushes to disk (0 = disabled)"),
     ]
 
 argslistheader = 'Arguments are:\n\n'
@@ -156,10 +177,11 @@ def download(self, params, filefunc, statusfunc, finfunc, errorfunc, doneflag, c
                           config['timeout'], ipv6_enable = config['ipv6_enabled'],
                           failfunc = failed, errorfunc = exchandler)
 
+    upnp_type = UPnP_test(config['upnp_nat_access'])
     try:
         listen_port = rawserver.find_and_bind(config['minport'], config['maxport'],
                         config['bind'], ipv6_socket_style = config['ipv6_binds_v4'],
-                        upnp = config['upnp_nat_access'])
+                        upnp = upnp_type, randomizer = config['random_port'])
     except socketerror, e:
         failed("Couldn't listen - " + str(e))
         return
@@ -179,7 +201,10 @@ def download(self, params, filefunc, statusfunc, finfunc, errorfunc, doneflag, c
     if pathFunc:
         pathFunc(d.getFilename())
 
-    if not d.initFiles(old_style = True):
+    hashcheck = d.initFiles(old_style = True)
+    if not hashcheck:
+        return
+    if not hashcheck():
         return
     if not d.startEngine():
         return
@@ -222,8 +247,8 @@ def parse_params(params, presets = {}):
     return config
 
 
-def get_usage(defaults = defaults, cols = 100):
-    return (argslistheader + formatDefinitions(defaults, cols))
+def get_usage(defaults = defaults, cols = 100, presets = {}):
+    return (argslistheader + formatDefinitions(defaults, cols, presets))
 
 
 def get_response(file, url, failfunc):
@@ -241,7 +266,10 @@ def get_response(file, url, failfunc):
             try:
                 h.seek(0)
             except:
-                h.close()
+                try:
+                    h.close()
+                except:
+                    pass
                 h = open(file, 'rb')
         else:
             try:
@@ -250,12 +278,14 @@ def get_response(file, url, failfunc):
                 failfunc(url+' bad url')
                 return None
         response = h.read()
-        h.close()
     
     except IOError, e:
         failfunc('problem getting response info - ' + str(e))
         return None
-    
+    try:    
+        h.close()
+    except:
+        pass
     try:
         response = bdecode(response)
         check_message(response)
@@ -268,7 +298,8 @@ def get_response(file, url, failfunc):
 
 class BT1Download:    
     def __init__(self, statusfunc, finfunc, errorfunc, excfunc, doneflag,
-                 config, response, infohash, id, rawserver, port):
+                 config, response, infohash, id, rawserver, port,
+                 appdataobj = None):
         self.statusfunc = statusfunc
         self.finfunc = finfunc
         self.errorfunc = errorfunc
@@ -289,6 +320,7 @@ class BT1Download:
         self.unpauseflag = Event()
         self.unpauseflag.set()
         self.storagewrapper = None
+        self.fileselector = None
         self.super_seeding_active = False
         self.filedatflag = Event()
         self.spewflag = Event()
@@ -296,9 +328,30 @@ class BT1Download:
         self.whenpaused = None
         self.finflag = Event()
         self.rerequest = None
-        self.rerequest_complete = lambda: None
-        self.rerequest_stopped = lambda: None
+        self.tcp_ack_fudge = config['tcp_ack_fudge']
+        
+        self.selector_enabled = config['selector_enabled']
+        if appdataobj:
+            self.appdataobj = appdataobj
+        elif self.selector_enabled:
+            self.appdataobj = ConfigDir()
+            self.appdataobj.deleteOldCacheData( config['expire_cache_data'],
+                                                [self.infohash] )
 
+        self.excflag = self.rawserver.get_exception_flag()
+        self.failed = False
+        self.checking = False
+        self.started = False
+
+
+    def checkSaveLocation(self, loc):
+        if self.info.has_key('length'):
+            return path.exists(loc)
+        for x in self.info['files']:
+            if path.exists(path.join(loc, x['path'][0])):
+                return True
+        return False
+                
 
     def saveAs(self, filefunc, pathfunc = None):
         try:
@@ -313,17 +366,17 @@ class BT1Download:
                 file = filefunc(self.info['name'], file_length,
                                 self.config['saveas'], False)
                 if file is None:
-                    return False
+                    return None
                 make(file)
                 files = [(file, file_length)]
             else:
-                file_length = 0
+                file_length = 0L
                 for x in self.info['files']:
                     file_length += x['length']
                 file = filefunc(self.info['name'], file_length,
                                 self.config['saveas'], True)
                 if file is None:
-                    return False
+                    return None
 
                 # if this path exists, and no files from the info dict exist, we assume it's a new download and 
                 # the user wants to create a new directory with the default name
@@ -331,7 +384,7 @@ class BT1Download:
                 if path.exists(file):
                     if not path.isdir(file):
                         self.errorfunc(file + 'is not a dir')
-                        return False
+                        return None
                     if len(listdir(file)) > 0:  # if it's not empty
                         for x in self.info['files']:
                             if path.exists(path.join(file, x['path'][0])):
@@ -343,7 +396,7 @@ class BT1Download:
                                     file = file[:-8]
                                 if path.exists(file) and not path.isdir(file):
                                     self.errorfunc("Can't create dir - " + self.info['name'])
-                                    return False
+                                    return None
                 make(file, True)
 
                 # alert the UI to any possible change in path
@@ -359,81 +412,162 @@ class BT1Download:
                     make(n)
         except OSError, e:
             self.errorfunc("Couldn't allocate dir - " + str(e))
-            return False
+            return None
 
         self.filename = file
         self.files = files
         self.datalength = file_length
 
-        return True
+        return file
     
 
     def getFilename(self):
         return self.filename
 
 
+    def _finished(self):
+        self.finflag.set()
+        try:
+            self.storage.set_readonly()
+        except (IOError, OSError), e:
+            self.errorfunc('trouble setting readonly at end - ' + str(e))
+        if self.superseedflag.isSet():
+            self._set_super_seed()
+        self.config['round_robin_period'] = max( self.config['round_robin_period'],
+            long( self.config['round_robin_period']
+                  * self.info['piece length']/200000) )
+        self.rerequest_complete()
+        self.finfunc()
+
+    def _data_flunked(self, amount, index):
+        self.ratemeasure_datarejected(amount)
+        if not self.doneflag.isSet():
+            self.errorfunc('piece %d failed hash check, re-downloading it' % index)
+
+    def _failed(self, reason):
+        self.failed = True
+        self.doneflag.set()
+        if reason is not None:
+            self.errorfunc(reason)
+        
+
     def initFiles(self, old_style = False, statusfunc = None):
         if self.doneflag.isSet():
             return None
         if not statusfunc:
             statusfunc = self.statusfunc
 
-        def failed(reason, self = self):
-            self.doneflag.set()
-            if reason is not None:
-                self.errorfunc(reason)
+        disabled_files = None
+        if self.selector_enabled:
+            self.priority = self.config['priority']
+            if self.priority:
+                try:
+                    self.priority = self.priority.split(',')
+                    assert len(self.priority) == len(self.files)
+                    self.priority = [int(p) for p in self.priority]
+                    for p in self.priority:
+                        assert p >= -1
+                        assert p <= 2
+                except:
+                    self.errorfunc('bad priority list given, ignored')
+                    self.priority = None
+
+            data = self.appdataobj.getTorrentData(self.infohash)
+            try:
+                disabled_files = [x == -1 for x in data['resume data']['priority']]
+            except:
+                try:
+                    disabled_files = [x == -1 for x in self.priority]
+                except:
+                    pass
+
         try:
             try:
-                self.storage = Storage(self.files, open, path.exists,
-                                       statusfunc, self.doneflag, self.config)
+                self.storage = Storage(self.files, self.info['piece length'],
+                                       self.doneflag, self.config, disabled_files)
             except IOError, e:
                 self.errorfunc('trouble accessing files - ' + str(e))
                 return None
             if self.doneflag.isSet():
                 return None
 
-            def finished(self = self):
-                self.finflag.set()
-                try:
-                    self.storage.set_readonly()
-                except (IOError, OSError), e:
-                    self.errorfunc('trouble setting readonly at end - ' + str(e))
-                if self.superseedflag.isSet():
-                    self._set_super_seed()
-                self.config['round_robin_period'] = max( self.config['round_robin_period'],
-                    int(self.config['round_robin_period']
-                        * self.info['piece length']/(200000)) )
-                self.rerequest_complete()
-                self.finfunc()
-
-            self.ratemeasure_datarejected = [lambda x: None]
-            def data_flunked(amount, index, self = self):
-                self.ratemeasure_datarejected(amount)
-                if not self.doneflag.isSet():
-                    self.errorfunc('piece %d failed hash check, re-downloading it' % index)
             self.storagewrapper = StorageWrapper(self.storage, self.config['download_slice_size'],
-                self.pieces, self.info['piece length'], finished, failed, statusfunc, self.doneflag,
-                self.config['check_hashes'], data_flunked, self.rawserver.external_add_task,
+                self.pieces, self.info['piece length'], self._finished, self._failed,
+                statusfunc, self.doneflag, self.config['check_hashes'],
+                self._data_flunked, self.rawserver.external_add_task,
                 self.config, self.unpauseflag)
-            if old_style:
-                self.storagewrapper.old_style_init()
             
         except ValueError, e:
-            failed('bad data - ' + str(e))
+            self._failed('bad data - ' + str(e))
         except IOError, e:
-            failed('IOError - ' + str(e))
+            self._failed('IOError - ' + str(e))
         if self.doneflag.isSet():
             return None
 
+        if self.selector_enabled:
+            self.fileselector = FileSelector(self.files, self.info['piece length'],
+                                             self.appdataobj.getPieceDir(self.infohash),
+                                             self.storage, self.storagewrapper,
+                                             self.rawserver.external_add_task,
+                                             self._failed)
+            if data:
+                data = data.get('resume data')
+                if data:
+                    self.fileselector.unpickle(data)
+                
+        self.checking = True
+        if old_style:
+            return self.storagewrapper.old_style_init()
         return self.storagewrapper.initialize
 
 
+    def getCachedTorrentData(self):
+        return self.appdataobj.getTorrentData(self.infohash)
+
+
+    def _make_upload(self, connection, ratelimiter, totalup):
+        return Upload(connection, ratelimiter, totalup,
+                      self.choker, self.storagewrapper, self.picker,
+                      self.config['max_slice_length'], self.config['max_rate_period'],
+                      self.config['upload_rate_fudge'], self.config['buffer_reads'])
+
+    def _kick_peer(self, connection):
+        def k(connection = connection):
+            connection.close()
+        self.rawserver.add_task(k,0)
+
+    def _ban_peer(self, ip):
+        self.encoder_ban(ip)
+
+    def _received_raw_data(self, x):
+        if self.tcp_ack_fudge:
+            x = int(x*self.tcp_ack_fudge)
+            self.ratelimiter.adjust_sent(x)
+#            self.upmeasure.update_rate(x)
+
+    def _received_data(self, x):
+        self.downmeasure.update_rate(x)
+        self.ratemeasure.data_came_in(x)
+
+    def _received_http_data(self, x):
+        self.downmeasure.update_rate(x)
+        self.ratemeasure.data_came_in(x)
+        self.downloader.external_data_received(x)
+
+    def _cancelfunc(self, pieces):
+        self.downloader.cancel_piece_download(pieces)
+        self.httpdownloader.cancel_piece_download(pieces)
+    def _reqmorefunc(self, pieces):
+        self.downloader.requeue_piece_download(pieces)
+
     def startEngine(self, ratelimiter = None, statusfunc = None):
         if self.doneflag.isSet():
             return False
         if not statusfunc:
             statusfunc = self.statusfunc
 
+        self.checking = False
+
         self.picker = PiecePicker(self.len_pieces, self.config['rarest_first_cutoff'],
                              self.config['rarest_first_priority_cutoff'])
         for i in xrange(self.len_pieces):
@@ -449,50 +583,59 @@ class BT1Download:
             self.ratelimiter = ratelimiter
         else:
             self.ratelimiter = RateLimiter(self.rawserver.add_task,
-                                           self.config['upload_unit_size'])
+                                           self.config['upload_unit_size'],
+                                           self.setConns)
             self.ratelimiter.set_upload_rate(self.config['max_upload_rate'])
         
-        def make_upload(connection, ratelimiter, totalup, self = self):
-            return Upload(connection, ratelimiter, totalup,
-                          self.choker, self.storagewrapper, self.picker,
-                          self.config['max_slice_length'], self.config['max_rate_period'],
-                          self.config['upload_rate_fudge'], self.config['buffer_reads'])
-        self.ratemeasure = RateMeasure(self.storagewrapper.get_amount_left())
+        self.ratemeasure = RateMeasure()
         self.ratemeasure_datarejected = self.ratemeasure.data_rejected
 
-        def kickpeer(connection, self = self):
-            def k(connection = connection):
-                connection.close()
-            self.rawserver.add_task(k,0)
-
-        self.encoder_ban = [lambda x: None]
-        def banpeer(ip, self = self):
-            self.encoder_ban(ip)
         self.downloader = Downloader(self.storagewrapper, self.picker,
             self.config['request_backlog'], self.config['max_rate_period'],
-            self.len_pieces, self.config['download_slice_size'], self.downmeasure,
-            self.config['snub_time'], self.config['auto_kick'],
-            kickpeer, banpeer, self.ratemeasure.data_came_in)
-
-        self.connecter = Connecter(make_upload, self.downloader, self.choker,
+            self.len_pieces, self.config['download_slice_size'],
+            self._received_data, self.config['snub_time'], self.config['auto_kick'],
+            self._kick_peer, self._ban_peer)
+        self.downloader.set_download_rate(self.config['max_download_rate'])
+        self.connecter = Connecter(self._make_upload, self.downloader, self.choker,
                             self.len_pieces, self.upmeasure, self.config,
                             self.ratelimiter, self.rawserver.add_task)
         self.encoder = Encoder(self.connecter, self.rawserver,
             self.myid, self.config['max_message_length'], self.rawserver.add_task,
-            self.config['keepalive_interval'], self.infohash, self.config)
+            self.config['keepalive_interval'], self.infohash,
+            self._received_raw_data, self.config)
         self.encoder_ban = self.encoder.ban
 
         self.httpdownloader = HTTPDownloader(self.storagewrapper, self.picker,
             self.rawserver, self.finflag, self.errorfunc, self.downloader,
-            self.config['max_rate_period'], self.infohash, self.downmeasure,
-            self.connecter.got_piece, self.ratemeasure.data_came_in)
+            self.config['max_rate_period'], self.infohash, self._received_http_data,
+            self.connecter.got_piece)
         if self.response.has_key('httpseeds') and not self.finflag.isSet():
             for u in self.response['httpseeds']:
                 self.httpdownloader.make_download(u)
 
+        if self.selector_enabled:
+            self.fileselector.tie_in(self.picker, self._cancelfunc, self._reqmorefunc)
+            if self.priority:
+                self.fileselector.set_priorities_now(self.priority)
+            self.appdataobj.deleteTorrentData(self.infohash)
+                                # erase old data once you've started modifying it
+        self.started = True
         return True
 
 
+    def rerequest_complete(self):
+        if self.rerequest:
+            self.rerequest.announce(1)
+
+    def rerequest_stopped(self):
+        if self.rerequest:
+            self.rerequest.announce(2)
+
+    def rerequest_lastfailed(self):
+        if self.rerequest:
+            return self.rerequest.last_failed
+        return False
+
     def startRerequester(self):
         if self.response.has_key('announce-list'):
             trackerlist = self.response['announce-list']
@@ -508,25 +651,13 @@ class BT1Download:
             self.errorfunc, self.excfunc, self.config['max_initiate'],
             self.doneflag, self.upmeasure.get_rate, self.downmeasure.get_rate)
 
-        def rerequest_complete(self = self):
-            self.rerequest.announce(1)
-        self.rerequest_complete = rerequest_complete
-
-        def rerequest_stopped(self = self):
-            self.rerequest.announce(2)
-        self.rerequest_stopped = rerequest_stopped
-
         self.rerequest.d(0)
 
 
     def _init_stats(self):
-        def lastfailedfunc(self=self):
-            if self.rerequest:
-                return self.rerequest.last_failed
-            return False
         self.statistics = Statistics(self.upmeasure, self.downmeasure,
-                    self.connecter, self.httpdownloader,
-                    lastfailedfunc, self.filedatflag)
+                    self.connecter, self.httpdownloader, self.ratelimiter,
+                    self.rerequest_lastfailed, self.filedatflag)
         if self.info.has_key('files'):
             self.statistics.set_dirstats(self.files, self.len_pieces, self.info['piece length'])
         if self.config['spew']:
@@ -539,7 +670,7 @@ class BT1Download:
         self._init_stats()
         DownloaderFeedback(self.choker, self.httpdownloader, self.rawserver.add_task,
             self.upmeasure.get_rate, self.downmeasure.get_rate,
-            self.ratemeasure.get_time_left, self.ratemeasure.get_size_left,
+            self.ratemeasure, self.storagewrapper.get_stats,
             self.datalength, self.finflag, self.spewflag, self.statistics,
             displayfunc, self.config['display_interval'])
 
@@ -547,7 +678,7 @@ class BT1Download:
         self._init_stats()
         d = DownloaderFeedback(self.choker, self.httpdownloader, self.rawserver.add_task,
             self.upmeasure.get_rate, self.downmeasure.get_rate,
-            self.ratemeasure.get_time_left, self.ratemeasure.get_size_left,
+            self.ratemeasure, self.storagewrapper.get_stats,
             self.datalength, self.finflag, self.spewflag, self.statistics)
         return d.gather
 
@@ -555,9 +686,22 @@ class BT1Download:
     def getPortHandler(self):
         return self.encoder
 
-    def shutdown(self):
-        self.storage.close()
-        self.rerequest_stopped()
+
+    def shutdown(self, torrentdata = {}):
+        if self.checking or self.started:
+            self.storagewrapper.sync()
+            self.storage.close()
+            self.rerequest_stopped()
+        if self.fileselector and self.started:
+            if not self.failed:
+                self.fileselector.finish()
+                torrentdata['resume data'] = self.fileselector.pickle()
+            try:
+                self.appdataobj.writeTorrentData(self.infohash,torrentdata)
+            except:
+                self.appdataobj.deleteTorrentData(self.infohash) # clear it
+        return not self.failed and not self.excflag.isSet()
+        # if returns false, you may wish to auto-restart the torrent
 
 
     def setUploadRate(self, rate):
@@ -582,6 +726,15 @@ class BT1Download:
         except AttributeError:
             pass
         
+    def setDownloadRate(self, rate):
+        try:
+            def s(self = self, rate = rate):
+                self.config['max_download_rate'] = rate
+                self.downloader.set_download_rate(rate)
+            self.rawserver.external_add_task(s)
+        except AttributeError:
+            pass
+
     def startConnection(self, ip, port, id):
         self.encrypter._start_connection((ip, port), id)
       
@@ -632,12 +785,12 @@ class BT1Download:
         return True
 
     def _pausemaker(self):
-        self.whenpaused = time()
+        self.whenpaused = clock()
         self.unpauseflag.wait()   # sticks a monkey wrench in the main thread
 
     def Unpause(self):
         self.unpauseflag.set()
-        if self.whenpaused and time()-self.whenpaused > 60:
+        if self.whenpaused and clock()-self.whenpaused > 60:
             def r(self = self):
                 self.rerequest.announce(3)      # rerequest automatically if paused for >60 seconds
             self.rawserver.external_add_task(r)
@@ -664,4 +817,10 @@ class BT1Download:
             if self.finflag.isSet():        # mode started when already finished
                 def r(self = self):
                     self.rerequest.announce(3)  # so after kicking everyone off, reannounce
-                self.rawserver.external_add_task(r)
\ No newline at end of file
+                self.rawserver.external_add_task(r)
+
+    def am_I_finished(self):
+        return self.finflag.isSet()
+
+    def get_transfer_stats(self):
+        return self.upmeasure.get_total(), self.downmeasure.get_total()
\ No newline at end of file
