@@ -1158,7 +1158,8 @@ class DownloadInfoFrame:
               'John Hoffman\n' +
               'Uoti Urpala\n' +
               'Jon Wolf\n' +
-              'Christoph Hohmann'))
+              'Christoph Hohmann\n' +
+              'Micah Anderson'))
             colSizer.Add (creditSizer, flag = wxALIGN_CENTER_HORIZONTAL)
             okButton = wxButton(panel, -1, 'Ok')
 #        okButton.SetFont(self.default_font)
@@ -2169,11 +2170,11 @@ class DownloadInfoFrame:
 class btWxApp(wxApp):
     def __init__(self, x, params):
         self.params = params
-        self.configfile = configReader()
         wxApp.__init__(self, x)
 
     def OnInit(self):
         doneflag = Event()
+        self.configfile = configReader()
         d = DownloadInfoFrame(doneflag, self.configfile)
         self.SetTopWindow(d.frame)
         if len(self.params) == 0:
